@@ -14,7 +14,8 @@ use Data::Dumper;
 use Wx qw(:everything);
 use base qw(Wx::Grid);
 use Gtk2::Ex::DbLinker::DatasheetHelper;
-use Wx::Event qw( EVT_GRID_CELL_CHANGED EVT_COMBOBOX EVT_GRID_RANGE_SELECT EVT_GRID_SELECT_CELL);
+use Wx::Event
+    qw( EVT_GRID_CELL_CHANGED EVT_COMBOBOX EVT_GRID_RANGE_SELECT EVT_GRID_SELECT_CELL);
 
 my %signals = ( 'Wx::GridCellChoiceEditor' => \&EVT_COMBOBOX, );
 
@@ -26,15 +27,15 @@ my %render = (
     combo   => sub { return Wx::GridCellChoiceEditor->new(@_); },
     boolean => sub { return Wx::GridCellBoolRenderer->new; },
     time    => sub { return Wx::GridCellDateTimeRenderer->new; },
-    
+
 );
 
 use constant {
-    UNCHANGED => 0,
-    CHANGED   => 1,
-    INSERTED  => 2,
-    DELETED   => 3,
-    LOCKED    => 4,
+    UNCHANGED     => 0,
+    CHANGED       => 1,
+    INSERTED      => 2,
+    DELETED       => 3,
+    LOCKED        => 4,
     STATUS_COLUMN => -1,
 };
 
@@ -52,12 +53,13 @@ sub new {
     $self->{dman} = $arg{data_manager};
 
     #$self->{fields} = $$req{fields}  || undef;
-    $self->{fields}        = $arg{fields};
-    $self->{on_changed}    = $arg{on_changed};      # Code that runs when a record is changed ( any column )
+    $self->{fields}     = $arg{fields};
+    $self->{on_changed} = $arg{on_changed}
+        ;    # Code that runs when a record is changed ( any column )
     $self->{on_row_select} = $arg{on_row_select};
     $self->{after_update}  = $arg{after_update};
 
-    #$self->{borders_size} = $$req{borders_size} || [22, 20]; #array ref of row1 height and col1 width
+#$self->{borders_size} = $$req{borders_size} || [22, 20]; #array ref of row1 height and col1 width
     $self->{borders_size} = $arg{borders_size};
 
     $self->{log} = Log::Log4perl->get_logger(__PACKAGE__);
@@ -70,30 +72,31 @@ sub new {
     $self->{log}->debug( "new called - cols: " . join( " ", @cols ) );
 
     # $self->_setup_fields;
-      
 
     #my $table =
     #$table->SetView($self);
     #$self->SetTable($table);
 
     $self->{ds_helper} = Gtk2::Ex::DbLinker::DatasheetHelper->new(
-        cols       => $self->{cols},
-        dman       => $self->{dman},
-	#col_number => sub { $self->colnumber_from_name( $_[0] ); }
+        cols => $self->{cols},
+        dman => $self->{dman},
+
+        #col_number => sub { $self->colnumber_from_name( $_[0] ); }
 
     );
-  ( $self->{fields}, $self->{hiddencols}) = 
-       $self->{ds_helper}->setup_fields(
-	    allfields=> $self->{fields}, 
-	    cols=> $self->{cols}
-            ) ;
-	    
-    my $table = $self->_setup_gridtable; 
+    ( $self->{fields}, $self->{hiddencols} ) =
+        $self->{ds_helper}->setup_fields(
+        allfields => $self->{fields},
+        cols      => $self->{cols}
+        );
+
+    my $table = $self->_setup_gridtable;
     $self->SetTable($table);
     $self->_setup_grid;
     $self->SetSelectionMode(wxGridSelectRows);
 
-    EVT_GRID_CELL_CHANGED( $self, sub { my $self = shift; $self->_changed(@_); } );
+    EVT_GRID_CELL_CHANGED( $self,
+        sub { my $self = shift; $self->_changed(@_); } );
     EVT_GRID_SELECT_CELL( $self, sub { shift->_cell_clicked(@_); } );
     EVT_GRID_RANGE_SELECT( $self, sub { shift->_row_selected(@_); } );
     $self->{log}->debug( "constructor done: rows: " . $self->GetNumberRows );
@@ -106,7 +109,8 @@ sub new {
 
 sub _setup_gridtable {
     my $self = shift;
-    return Wx::Perl::DbLinker::DBGridTable->new( { dman => $self->{dman}, fields => $self->{fields}, grid => $self } );
+    return Wx::Perl::DbLinker::DBGridTable->new(
+        { dman => $self->{dman}, fields => $self->{fields}, grid => $self } );
 
 }
 
@@ -128,15 +132,19 @@ sub _setup_grid {
 
         if ( $field->{renderer} eq "toggle" ) {
             $field->{editor} = $render{ $field->{renderer} }();
-        } elsif ( $field->{renderer} eq "combo" ) {
+        }
+        elsif ( $field->{renderer} eq "combo" ) {
 
-            $field->{editor} = $render{ $field->{renderer} }( $self->_setup_combo( $field->{name} ), 0 );
-            $self->{log}->debug( "field name with combo renderer: " . $field->{name} );
+            $field->{editor} = $render{ $field->{renderer} }
+                ( $self->_setup_combo( $field->{name} ), 0 );
+            $self->{log}->debug(
+                "field name with combo renderer: " . $field->{name} );
 
             my $signal = $signals{ ref $field->{editor} };
 
             #http://sourceforge.net/p/wxperl/mailman/message/8872987/
-            &$signal( $self, -1, sub { shift; $self->_combo_edited( $field->{name}, @_ ); } );
+            &$signal( $self, -1,
+                sub { shift; $self->_combo_edited( $field->{name}, @_ ); } );
 
             #  $renderer->{col_data} = $lastcol++ ;
             #
@@ -158,9 +166,12 @@ sub _setup_grid {
 				#	Gtk2::TreeViewColumn->new_with_attributes($field->{name}, $renderer, 'text' => $renderer->{col_data} );
 =cut
 
-        } else {
-            $self->{log}->debug( "field name with txt renderer: " . $field->{name} );
-            $field->{editor} = $render{ $field->{renderer} }() if ($render{ $field->{renderer} });
+        }
+        else {
+            $self->{log}
+                ->debug( "field name with txt renderer: " . $field->{name} );
+            $field->{editor} = $render{ $field->{renderer} }()
+                if ( $render{ $field->{renderer} } );
 
             if ( grep /^$field->{name}$/, @apk ) {
                 $self->{log}->debug("not editable because it's a pk");
@@ -174,15 +185,17 @@ sub _setup_grid {
          # die ($self->GetNumberRows);
          # enlever -2 a cause des labels
     for my $r ( 0 .. $self->GetNumberRows() - 1 ) {    # Row Header Text
-                                                       #my $rptr = $r+1;
-                                                       #$grid->SetRowLabelValue($r, "Row $rptr");
-                                                       #$grid->SetRowLabelBackgroundColour($r, wxGREEN);
+            #my $rptr = $r+1;
+            #$grid->SetRowLabelValue($r, "Row $rptr");
+            #$grid->SetRowLabelBackgroundColour($r, wxGREEN);
         $self->SetRowLabelValue( $r, (STATUS_LAB)[UNCHANGED] );
 
         #$grid->SetReadOnly($r, 0);
         for my $field ( @{ $self->{fields} } ) {
-		#my $col = $self->{colname_to_number}->{ $field->{name} };
-		my $col = $self->{ds_helper}->colnumber_from_name( $field->{name} );
+
+            #my $col = $self->{colname_to_number}->{ $field->{name} };
+            my $col =
+                $self->{ds_helper}->colnumber_from_name( $field->{name} );
             if ( $r == 0 ) {
 
                 my $label = (
@@ -192,7 +205,7 @@ sub _setup_grid {
                 );
                 $self->{log}->debug( "col: ", $col, " label: ", $label );
                 $self->{log}->debug( "cell editor: " . ref $field->{editor} );
-		die unless defined $col;
+                croak("\$col undef") unless defined $col;
                 $self->SetColLabelValue( $col, $label );
                 if ( defined $field->{size} ) {
                     $self->SetColSize( $col, $field->{size} );    #if ($r==0);
@@ -202,9 +215,9 @@ sub _setup_grid {
             if ( $ed_ref eq "Wx::GridCellChoiceEditor" ) {
                 $self->SetCellEditor( $r, $col, $field->{editor} );
 
-                # remplacer la valeur de id_credit par le text correspondant dans la combo
-                # _find_index est appele pour toutes les lignes de la col, si la liste est longue, ce n'est pas efficace
-                # $id est la valeur affichee dans la grid qui provient de la table
+# remplacer la valeur de id_credit par le text correspondant dans la combo
+# _find_index est appele pour toutes les lignes de la col, si la liste est longue, ce n'est pas efficace
+# $id est la valeur affichee dans la grid qui provient de la table
                 my $id = $self->GetCellValue( $r, $col );
 
                 # $val est le array ref des id
@@ -213,37 +226,40 @@ sub _setup_grid {
                 #$pos et l'index de id dans ce vecteur
                 my $pos = $self->_find_index( $id, @$val );
 
-                # $text est le text a afficher dans la cellule  la place du id
+               # $text est le text a afficher dans la cellule  la place du id
 
                 my $text;
                 if ( $pos > -1 ) {
                     $text = $self->{combo_val}->{ $field->{name} }->[$pos];
-                } else {
+                }
+                else {
                     $text = "";
                 }
 
-                # combo_cur contient l'id de la derniere ligne affichee de la grid
-                # $self->{combo_cur}->{$field->{name}} = $self->{combo_id}->{ $field->{name} }->[$pos];
-                #$self->{log}->debug( $field->{name}, " id: ", $id, " text: ", $text);
-                #$self->{combo_cur}->{ $field->{name} } est un array ref des id sous-jacents aux lignes affichees
-                #my $aref = $self->{combo_cur}->{ $field->{name} };
+# combo_cur contient l'id de la derniere ligne affichee de la grid
+# $self->{combo_cur}->{$field->{name}} = $self->{combo_id}->{ $field->{name} }->[$pos];
+#$self->{log}->debug( $field->{name}, " id: ", $id, " text: ", $text);
+#$self->{combo_cur}->{ $field->{name} } est un array ref des id sous-jacents aux lignes affichees
+#my $aref = $self->{combo_cur}->{ $field->{name} };
                 my $aref = $self->{combo_cur};
                 push @$aref, $id;
                 $self->SetCellOverflow( $r, $col, 0 );
                 $self->SetCellValue( $r, $col, $text );
 
                 # $self->GetCellEditor($r,$col)->StartingClick;
-            } else {
+            }
+            else {
 
                 $self->SetCellRenderer( $r, $col, $field->{editor} );
                 if ( $field->{renderer_function} ) {
                     my $coderef = $field->{renderer_function};
-                    $self->SetCellValue( $r, $col, &$coderef( $r, $col, $self->GetTable ) );
+                    $self->SetCellValue( $r, $col,
+                        &$coderef( $r, $col, $self->GetTable ) );
                 }
 
             }
 
-            #$self->{log}->debug("set cell renderer ", $ed_ref, " row: ", $r, " col: ", $col);
+#$self->{log}->debug("set cell renderer ", $ed_ref, " row: ", $r, " col: ", $col);
 
             if ( $field->{editor_ro} ) {
                 $self->SetReadOnly( $r, $col, 1 );
@@ -264,10 +280,15 @@ sub _setup_grid {
 
 sub _setup_combo {
     my ( $self, $fieldname ) = @_;
-     my $column_no =  $self->colnumber_from_name($fieldname);
-    my $fields = $self->{fields}[$column_no ];
-    my $col_ref = $self->{ds_helper}->init_combo_setup(name=> $fieldname, fields => $fields);	
-    my ( $rowsref, $idsref ) = $self->{ds_helper}->setup_combo( fields => $fields, name => $fieldname, col_ref=>$col_ref);
+    my $column_no = $self->colnumber_from_name($fieldname);
+    my $fields    = $self->{fields}[$column_no];
+    my $col_ref   = $self->{ds_helper}
+        ->init_combo_setup( name => $fieldname, fields => $fields );
+    my ( $rowsref, $idsref ) = $self->{ds_helper}->setup_combo(
+        fields  => $fields,
+        name    => $fieldname,
+        col_ref => $col_ref
+    );
 
     $self->{combo_id}->{$fieldname}  = $idsref;
     $self->{combo_val}->{$fieldname} = $rowsref;
@@ -283,7 +304,8 @@ sub get_column_value {
     my ( $self, $sql_fieldname ) = @_;
 
     my $current_row = $self->{current_row};
-    $self->{log}->debug( "get_column_value current row : " . ( defined $current_row ? $current_row : " undef" ) );
+    $self->{log}->debug( "get_column_value current row : "
+            . ( defined $current_row ? $current_row : " undef" ) );
 
     # my $model = $self->{treeview}->get_model;
 
@@ -298,8 +320,10 @@ sub get_column_value {
         my $aref = $self->{combo_cur};
         $value = $aref->[$current_row];
         $self->{log}->debug( "get_column_value combo_cur : ", Dumper $aref);
-        $self->{log}->debug( "get_column_value is combo return  : " . $value );
-    } else {
+        $self->{log}
+            ->debug( "get_column_value is combo return  : " . $value );
+    }
+    else {
 
         $value = $self->GetCellValue( $current_row, $column_no );
     }
@@ -313,13 +337,14 @@ sub set_column_value {
 
     my $done;
 
-    #if ( $self->{mult_select} ) {
-    #    $self->{log}->debug("set_column_value called with multi_select enabled -> setting value in 1st selected row");
-    #}
+#if ( $self->{mult_select} ) {
+#    $self->{log}->debug("set_column_value called with multi_select enabled -> setting value in 1st selected row");
+#}
 
     #my  @selected_rows = $self->GetSelectedRows;
     my $current_row = $self->{current_row};
-    $self->{log}->debug( "set_column_value current row : " . ( defined $current_row ? $current_row : " undef" ) );
+    $self->{log}->debug( "set_column_value current row : "
+            . ( defined $current_row ? $current_row : " undef" ) );
 
     #if ( ! scalar( @selected_rows ) ) {
     #    return 0;
@@ -332,7 +357,8 @@ sub set_column_value {
 
         $self->{log}->debug( $fieldname . " is a combo : no update done" );
         $done = 0;
-    } else {
+    }
+    else {
 
         $self->SetCellValue( $current_row, $col, $value );
         $done = 1;
@@ -349,6 +375,7 @@ sub get_current_row {
 sub colnumber_from_name {
 
     my ( $self, $fieldname ) = @_;
+
     #confess "fieldname undef" unless ( defined $fieldname );
     #return $self->{colname_to_number}->{$fieldname}
     return $self->{ds_helper}->colnumber_from_name($fieldname);
@@ -389,8 +416,8 @@ sub update {
     #$self->ForceRefresh;
     my $table = $self->_setup_gridtable;
 
-    #	Wx::Perl::DbLinker::DBGridTable->new({dman=> $self->{dman}, fields => $self->{fields}, grid=>$self,});
-    #$self->{log}->debug("update : ",join(" ", @));
+#	Wx::Perl::DbLinker::DBGridTable->new({dman=> $self->{dman}, fields => $self->{fields}, grid=>$self,});
+#$self->{log}->debug("update : ",join(" ", @));
     $self->SetTable($table);
     $self->_setup_grid;
     if ( defined $self->{after_update} ) {
@@ -407,36 +434,60 @@ sub apply {
 
     #$self->{ds_helper}->init_rows($self);
     $self->{ds_helper}->init_apply(
-        iter         => 0,
-        next         => sub { my $p = $_[0]; $p++; return ( $p < $last ? $p : undef ); },
-        has_more_row => sub { my $v = ( $self->{iter} < $last ); $self->{log}->debug( "has_more_row ", ( $v ? " true" : " false" ) ); return $v; },
-        get_val      => sub {
+        iter => 0,
+        next =>
+            sub { my $p = $_[0]; $p++; return ( $p < $last ? $p : undef ); },
+        has_more_row => sub {
+            my $v = ( $self->{iter} < $last );
+            $self->{log}
+                ->debug( "has_more_row ", ( $v ? " true" : " false" ) );
+            return $v;
+        },
+        get_val => sub {
             my ( $row, $col, $name ) = @_;
+            #$name can be undef in the case of $col = -1
             my $val;
-	    if ( $col == STATUS_COLUMN ) {
+            if ( $col == STATUS_COLUMN ) {
                 my $status_lab = $self->GetRowLabelValue($row);
 
                 $val = $self->_find_index( $status_lab, (STATUS_LAB) );
-	     } else {
+            }
+            else {
+              $self->{log}->debug(Dumper $self->{combo_cur}); #->{$name}
                 if ( $self->_is_combo($col) ) {
-                    my $aref = $self->{combo_cur}->{$name};
+                    #my $aref = $self->{combo_cur}->{$name};
+                    my $aref = $self->{combo_cur};
                     $val = $aref->[$row];
-                    $self->{log}->debug( " found " . $val . " for grid row " . $row . " for combo " . $name );
-                } else {
-                    $val = $self->GetTable->GetValue( $row, $col );
+                    $self->{log}->debug( " found "
+                            . $val
+                            . " for grid row "
+                            . $row
+                            . " for combo "
+                            . $name );
                 }
-	     }
-            $self->{log}->debug( "get_val found: ", ( defined $val ? $val : "undef" ), " for row : ", $row, " col : ", $col );
+                else {
+                    $val = $self->GetTable->GetValue( $row, $col );
+                    $val = undef unless (defined $val && length($val)); #Wx grid return an empty string -> insert fails with DbiDM
+                }
+            }
+            $self->{log}->debug(
+                "get_val found: ",
+                ( defined $val ? "*" . $val ."*" : "undef" ),
+                " for row : ", $row, " col : ", $col
+            );
             return $val;
         },
         set_val => sub {
             my ( $row, $col, $val ) = @_;
-            $self->{log}->debug( "set_val : ", ( defined $val ? $val : " undef" ), " for row : ", $row, " col : ", $col );
+            $self->{log}
+                ->debug( "set_val : ", ( defined $val ? $val : " undef" ),
+                " for row : ", $row, " col : ", $col );
             if ( $col == STATUS_COLUMN ) {
                 my $label = (STATUS_LAB)[$val];
                 $self->{log}->debug( "label found : ", $label );
                 $self->SetRowLabelValue( $row, $label );
-            } else {
+            }
+            else {
                 $self->SetCellValue( $row, $col, $val );
             }
         },
@@ -444,7 +495,7 @@ sub apply {
             $self->{log}->debug( "deleting at row ", $_[0] );
             $self->DeleteRows( $_[0] );
         },
-	status_col => STATUS_COLUMN,
+        status_col => STATUS_COLUMN,
     );
 
     #$self->{ds_helper}->{dman} = $self->{dman};
@@ -459,7 +510,8 @@ sub insert {
     my ( $self, @columns_and_values ) = @_;
     $self->{log}->debug("insert");
 
-    $self->{log}->debug( "new rec default values: " . join( " ", @columns_and_values ) );
+    $self->{log}->debug(
+        "new rec default values: " . join( " ", @columns_and_values ) );
 
     $self->AppendRows(1);
     my $last = $self->GetNumberRows - 1;
@@ -474,8 +526,14 @@ sub insert {
             if ($i) {
                 $col_no = $col_no_or_value;
                 $i      = 0;
-            } else {
-                $self->{log}->debug( "last: " . $last . " col: " . $col_no . " value: " . $col_no_or_value );
+            }
+            else {
+                $self->{log}->debug( "last: "
+                        . $last
+                        . " col: "
+                        . $col_no
+                        . " value: "
+                        . $col_no_or_value );
                 $self->SetCellValue( $last, $col_no, $col_no_or_value );
                 $i = 1;
             }
@@ -489,7 +547,7 @@ sub delete {
     my $self = shift;
     $self->{log}->debug("delete");
     $self->SetRowLabelValue( $self->{current_row}, (STATUS_LAB)[DELETED] )
-      if ( defined $self->{current_row} );
+        if ( defined $self->{current_row} );
 
 }
 
@@ -513,7 +571,8 @@ sub _row_selected {
     my ( $self, $e ) = @_;
     my $top = $e->GetTopRow;
     my $bot = $e->GetBottomRow;
-    $self->{log}->debug( "_row_selected top row: " . $top . " bottom row : " . $bot );
+    $self->{log}
+        ->debug( "_row_selected top row: " . $top . " bottom row : " . $bot );
     if ( $top == $bot ) {
         $self->{current_row} = $top;
         $self->{on_row_select}() if ( defined $self->{on_row_select} );
@@ -548,7 +607,13 @@ sub _combo_edited {
     # die(Dumper $aref);
     my @a = @{$aref};
     $self->{log}->debug( "id: " . Dumper($id) );
-    $self->{log}->debug( "new value:  " . $new_text . " index: " . $i . " corresponding id : " . $a[$i] );
+    $self->{log}->debug( "new value:  "
+            . $new_text
+            . " index: "
+            . $i
+            . " corresponding id : "
+            . $a[$i] );
+
     #$self->{combo_cur}->{$id } = $a[$i];
     #update the array backing the combo column in the grid
     #$self->{combo_cur}->{$id}->[$current_row] = $a[$i];
@@ -567,7 +632,7 @@ sub _find_index {
 
     #my( $found, $index ) = ( undef, -1 );
     my $index = -1;
-    for ( my $i = 0 ; $i < @array ; $i++ ) {
+    for ( my $i = 0; $i < @array; $i++ ) {
         if ( $array[$i] eq $what ) {
 
             # $found = $array[$i];
@@ -613,10 +678,11 @@ sub new {
     my $self = $class->SUPER::new;
     $self->{log} = Log::Log4perl->get_logger(__PACKAGE__);
     $self->{log}->debug("new called");
-    $self->{dman}    = $args->{dman};
-    $self->{fields}  = $args->{fields};
-    $self->{grid}    = $args->{grid};
-    $self->{cols}    = scalar @{ $self->{fields} };  #here cols holds the number of fields ... in Wxdatasheet it's an ar of the fields names
+    $self->{dman}   = $args->{dman};
+    $self->{fields} = $args->{fields};
+    $self->{grid}   = $args->{grid};
+    $self->{cols}   = scalar @{ $self->{fields} }
+        ; #here cols holds the number of fields ... in Wxdatasheet it's an ar of the fields names
     $self->{coldata} = [];
     $self->{array}   = [];
     $self->{rows}    = $self->{dman}->row_count();
@@ -629,7 +695,7 @@ sub new {
             my $fname = $f->{name};
             my $v     = $self->{dman}->get_field($fname);
 
-            #$self->{log}->debug(" row: ", $i , " $fname: ", (defined $v ? $v : " undef"));
+#$self->{log}->debug(" row: ", $i , " $fname: ", (defined $v ? $v : " undef"));
             $$self{array}->[$i][ $col++ ] = $v;
         }
 
@@ -648,8 +714,8 @@ sub GetNumberCols {
 sub GetNumberRows {
     my ($self) = @_;
 
-    #$$self{log}->debug("GrigTable-GNR called");
-    #mais il y a la ligne des labels...mais grid->GetNumberRows ne la compte pas non plus
+#$$self{log}->debug("GrigTable-GNR called");
+#mais il y a la ligne des labels...mais grid->GetNumberRows ne la compte pas non plus
     return $self->{rows};
 }
 
@@ -667,18 +733,21 @@ sub GetValue {
     if ( $row < $self->GetNumberRows && $col < $self->GetNumberCols ) {
         my $aref = $$self{array}->[$row];
 
-        #$self->{log}->debug("GetValue : " . $row . " " . $col . " array : " . (defined $aref ? " defined " : " undef") );
-        if ( defined $$self{array}->[$row] ) {    #retourner $result s'il a ete attribue par SetValue
+#$self->{log}->debug("GetValue : " . $row . " " . $col . " array : " . (defined $aref ? " defined " : " undef") );
+        if ( defined $$self{array}->[$row] )
+        {    #retourner $result s'il a ete attribue par SetValue
             $result = $$self{array}->[$row][$col];
-        } else {
+        }
+        else {
             confess("array undef");
         }
 
         my $fname = $self->{fields}[$col]->{name};
 
-        #$self->{log}->debug("GetValue: ", (defined $result? $result: " undef"), " Field: ", $fname, " row: ", $row, " col: ", $col);
+#$self->{log}->debug("GetValue: ", (defined $result? $result: " undef"), " Field: ", $fname, " row: ", $row, " col: ", $col);
 
-    } else {
+    }
+    else {
         carp( "Index out of bound in DBGridTable: " . $row . " : " . $col );
     }
 
@@ -691,24 +760,25 @@ sub GetValue {
 sub SetValue {
     my ( $self, $row, $col, $value ) = @_;
 
-    #$self->{log}->debug ("SetValue GNR: " . $self->GetNumberRows . " pos to add to is " . $row) ;
+#$self->{log}->debug ("SetValue GNR: " . $self->GetNumberRows . " pos to add to is " . $row) ;
     confess("Array out of bounds too much rows")
-      unless ( $row < $self->GetNumberRows );
+        unless ( $row < $self->GetNumberRows );
     confess("Array out of bounds too much cols")
-      unless ( $col < $self->GetNumberCols );
+        unless ( $col < $self->GetNumberCols );
 
     my $fname = $self->{fields}->[$col]->{name};
 
-    #$self->{log}->debug("SetValue row : ", $row, " col: ", $col, " field: ", $fname , " value: ", (defined $value ? $value : " undef"));
+#$self->{log}->debug("SetValue row : ", $row, " col: ", $col, " field: ", $fname , " value: ", (defined $value ? $value : " undef"));
 
-    #fait dans apply ... !
-    #D'autant plus que $value est la valeur affichee par un combo et non l'id correspondant
-    #$self->{dman}->set_row_pos($row);
-    #    $self->{dman}->set_field($fname, $value);
+#fait dans apply ... !
+#D'autant plus que $value est la valeur affichee par un combo et non l'id correspondant
+#$self->{dman}->set_row_pos($row);
+#    $self->{dman}->set_field($fname, $value);
 
     $$self{array}->[$row][$col] = $value;
 }
 
+=for comment
 sub GetPlData_todel {
     my ( $self, $row, $col ) = @_;
     return undef
@@ -737,6 +807,7 @@ sub CanGetValueAs_todel {
     my $self = shift;
     return $self->CanSetValueAs(@_);
 }
+=cut
 
 sub SetColLabelValue {
     my ( $self, $col, $value ) = @_;
@@ -748,7 +819,9 @@ sub SetColLabelValue {
 sub _checkCol {
     my ( $self, $col ) = @_;
     my $cols = $self->GetNumberCols;
-    return undef unless defined $col && abs($col) < $cols;
+
+    #return undef unless defined $col && abs($col) < $cols;
+    return unless defined $col && abs($col) < $cols;
     return $cols + $col if $col < 0;
     return $col;
 }
@@ -756,7 +829,9 @@ sub _checkCol {
 sub GetColLabelValue {
     my ( $self, $col ) = @_;
     $col = $self->_checkCol($col);
-    return undef unless defined $col;
+
+    #return undef unless defined $col;
+    return unless defined $col;
     return $$self{coldata}->[$col]->{label};
 }
 
@@ -770,14 +845,18 @@ sub SetRowLabelValue {    # Modeled after the wiki for custom labels
 sub GetRowLabelValue {    # Modeled after the wiki for custom labels
     my ( $grid, $row ) = @_;
     $row = $grid->_checkRow($row);
-    return undef unless defined $row;
+
+    #return undef unless defined $row;
+    return unless defined $row;
     return $$grid{rowdata}->[$row]->{label};
 }
 
 sub _checkRow {           # Modeled after the wiki for custom labels
     my ( $grid, $row ) = @_;
     my $rows = $grid->GetNumberRows;
-    return undef unless defined $row && abs($row) < $rows;
+
+    #return undef unless defined $row && abs($row) < $rows;
+    return unless defined $row && abs($row) < $rows;
     return $rows + $row if $row < 0;
     return $row;
 }
@@ -785,7 +864,9 @@ sub _checkRow {           # Modeled after the wiki for custom labels
 sub GetColLabelWidth {
     my ( $self, $col ) = @_;
     $col = $self->_checkCol($col);
-    return undef unless defined $col;
+
+    #return undef unless defined $col;
+    return unless defined $col;
     return $$self{coldata}->[$col]->{width};
 }
 
@@ -811,6 +892,7 @@ sub GetView {
     return $self->{grid};
 }
 
+=for comment
 sub SetAttr_todel {
     my ( $self, $attr, $row, $col ) = @_;
     $row = $self->_checkRow($row);
@@ -819,6 +901,7 @@ sub SetAttr_todel {
     $$self{array}->[$row][$col]->SetAttr($attr);
     return;
 }
+=cut
 
 sub SetColAttr {
     my ( $self, $attr, $col ) = @_;
@@ -828,6 +911,7 @@ sub SetColAttr {
     return;
 }
 
+=for comment
 sub GetAttr_todel {
     my ( $self, $row, $col ) = @_;
     $row = $self->_checkRow($row);
@@ -836,6 +920,7 @@ sub GetAttr_todel {
     my $attr = $$self{array}->[$row][$col]->GetAttr;
     return defined $attr ? $attr : $self->GetColAttr($col);
 }
+=cut
 
 sub GetColAttr {
     my ( $self, $col ) = @_;
@@ -856,7 +941,7 @@ sub AppendRows {
         # en partant de la premiere
         my $last = $pos + $rows_add;
 
-        for ( my $row = $pos ; $row < $last ; $row++ ) {
+        for ( my $row = $pos; $row < $last; $row++ ) {
 
             #  creer des cellules vides
             $$self{array}->[$row] = $self->_newRow($row);
@@ -865,7 +950,9 @@ sub AppendRows {
 
         if ( my $grid = $self->GetView ) {
             $self->{log}->debug("AppendRows");
-            my $msg = Wx::GridTableMessage->new( $self, wxGRIDTABLE_NOTIFY_ROWS_APPENDED, $rows_add );
+            my $msg =
+                Wx::GridTableMessage->new( $self,
+                wxGRIDTABLE_NOTIFY_ROWS_APPENDED, $rows_add );
             $grid->ProcessTableMessage($msg);
         }
     };
@@ -880,7 +967,7 @@ sub _newRow {
     my ( $self, $pos ) = @_;
     $self->{log}->debug("_newRow");
     my $row = [];
-    for ( my $col = 0 ; $col < $$self{cols} ; $col++ ) {
+    for ( my $col = 0; $col < $$self{cols}; $col++ ) {
 
         #    $$row[$col] = DBGridCell->new;
         $$row[$col] = undef;
@@ -894,25 +981,28 @@ sub _newRow {
 sub DeleteRows {
     my ( $self, $pos, $rows ) = @_;
     return 0
-      unless defined $rows
-      && $rows >= 0
-      && defined $pos
-      && $pos >= 0
-      && $pos < $$self{rows};
+        unless defined $rows
+        && $rows >= 0
+        && defined $pos
+        && $pos >= 0
+        && $pos < $$self{rows};
     if ( $pos + $rows < $$self{rows} ) {
-        for ( my $row = $pos ; $row < $pos + $rows ; $row++ ) {
+        for ( my $row = $pos; $row < $pos + $rows; $row++ ) {
             delete $$self{array}->[$row];
 
             $$self{array}->[$row] = $$self{array}->[ $row + $pos ];
         }
         $$self{rows} -= $rows;
-    } else {
+    }
+    else {
         $rows = $$self{rows} - $pos if $$self{rows} < $pos + $rows;
         delete @{ $$self{array} }[ $pos .. $rows ];
         $$self{rows} -= $rows;
     }
     if ( my $grid = $self->GetView ) {
-        my $msg = Wx::GridTableMessage->new( $self, wxGRIDTABLE_NOTIFY_ROWS_DELETED, $pos, $rows );
+        my $msg =
+            Wx::GridTableMessage->new( $self, wxGRIDTABLE_NOTIFY_ROWS_DELETED,
+            $pos, $rows );
         $grid->ProcessTableMessage($msg);
     }
     return 1;
@@ -927,25 +1017,30 @@ sub setup_empty_row {
         if ( ref $field->{editor} eq "Wx::GridCellChoiceEditor" ) {
             $grid->SetCellEditor( $r, $col, $field->{editor} );
 
-            #my $id = $grid->GetCellValue($r, $col);
-            # $val est le array ref des id
-            #my $val = $grid->{combo_id}->{ $field->{name} };
-            #$pos et l'index de id dans ce vecteur
-            #my $pos = $self->_find_index($id, @$val);
-            # $text est le text a afficher dans la cellule  la place du id
-            #my $text = $self->{combo_val}->{$field->{name} }->[$pos];
-            # combo_cur contient l'id de la derniere ligne affichee de la grid
-            # $self->{combo_cur}->{$field->{name}} = $self->{combo_id}->{ $field->{name} }->[$pos];
-            # $self->{log}->debug( $field->{name}, " id: ", $id, " text: ", $text);
-            #$self->{combo_cur}->{ $field->{name} } est un array ref des id sous-jacents aux lignes affichees
-            #my $aref = $self->{combo_cur}->{ $field->{name} };
-            #push @$aref, $id;
+#my $id = $grid->GetCellValue($r, $col);
+# $val est le array ref des id
+#my $val = $grid->{combo_id}->{ $field->{name} };
+#$pos et l'index de id dans ce vecteur
+#my $pos = $self->_find_index($id, @$val);
+# $text est le text a afficher dans la cellule  la place du id
+#my $text = $self->{combo_val}->{$field->{name} }->[$pos];
+# combo_cur contient l'id de la derniere ligne affichee de la grid
+# $self->{combo_cur}->{$field->{name}} = $self->{combo_id}->{ $field->{name} }->[$pos];
+# $self->{log}->debug( $field->{name}, " id: ", $id, " text: ", $text);
+#$self->{combo_cur}->{ $field->{name} } est un array ref des id sous-jacents aux lignes affichees
+#my $aref = $self->{combo_cur}->{ $field->{name} };
+#push @$aref, $id;
             $grid->SetCellOverflow( $r, $col, 0 );
 
             #  $self->SetCellValue($r, $col, $text);
-        } else {
+        }
+        else {
             $grid->SetCellRenderer( $r, $col, $field->{editor} );
-            $self->{log}->debug( "set cell renderer ", ref $field->{editor}, " row: ", $r, " col: ", $col );
+            $self->{log}->debug(
+                "set cell renderer ",
+                ref $field->{editor},
+                " row: ", $r, " col: ", $col
+            );
 
         }
         if ( $field->{editor_ro} ) {
