@@ -1,5 +1,6 @@
 
 package Wx::Perl::DbLinker::Wxform;
+use Class::InsideOut qw(public private register id);
 use Wx::Perl::DbLinker;
 our $VERSION = $Wx::Perl::DbLinker::VERSION;
 
@@ -10,7 +11,7 @@ use Carp qw(croak confess carp);
 
 #use DateTime::Format::Strptime;
 use Data::Dumper;
-
+use Scalar::Util qw( weaken );
 use Wx qw[:everything];
 
 #EVT_DATAVIEW_ITEM_VALUE_CHANGED
@@ -72,6 +73,16 @@ my %getter = (
 
 );
 
+private log => my %log;
+private event => my %events;
+private states => my %states;
+private widgets => my %widgets;
+private fields_with_event => my %ecols;
+private is_with_event => my %is_ecols;
+private datawidgets_value => my %datawidgets_value;
+private datawidgets_pos => my %datawidgets_pos;
+private combos => my %combos;
+
 sub new {
     my $class = shift;
 
@@ -86,38 +97,79 @@ sub new {
     );
 
     my %arg = ( ref $_[0] eq "HASH" ? ( %def, %{ $_[0] } ) : ( %def, @_ ) );
+   
     my $self = $class->SUPER::new(
         childclass           => __PACKAGE__,
+         data_manager => $arg{data_manager},
+        builder => $arg{builder},
+        datawidgets => $arg{datawidgets},
+        datawidgets_ro => $arg{datawidgets_ro},
+        on_current => $arg{on_current},
+        date_formatters => $arg{date_formatters},
+        datawidgets_ro => $arg{datawidgets_ro},
+        time_zone => $arg{time_zone},
+        locale => $arg{locale},
         rec_spinner_callback => sub {
             my $self = shift;
-            return unless $self->{rec_spinner};
-            $self->{rec_spinner}->SetValue( $self->{pos} + 1 );
+             my $id = id $self;
+            return unless $widgets{ $id }->{rec_spinner};
+            $widgets{ $id }->{rec_spinner}->SetValue( $self->_pos + 1 );
         },
         rec_spinner_insert_callback => sub {
             my ( $self, $new_pos ) = @_;
-            return unless $self->{rec_spinner};
-            my $first = ( $self->{pos} < 0 ? 0 : 1 );
-            $self->{rec_spinner}->SetRange( $first, $new_pos + 1 );
-            $self->{rec_spinner}->SetValue( $new_pos + 1 );
+             my $id = id $self;
+            return unless $widgets{ $id }->{rec_spinner};
+            my $first = ( $self->_pos < 0 ? 0 : 1 );
+            $widgets{ $id }->{rec_spinner}->SetRange( $first, $new_pos + 1 );
+            $widgets{ $id }->{rec_spinner}->SetValue( $new_pos + 1 );
         },
     );
-    @$self{qw(dman cols ecols)} =
-      delete @arg{qw(data_manager datawidgets datawidgets_changed)};
-    @$self{ keys %arg } = values(%arg);
-    bless $self, $class;
+    register $self;
+     my $ido = id $self;
+      delete @arg{ $self->_super_args_needed };
+      # @$self{qw(dman cols ecols)} = delete @arg{qw(data_manager datawidgets datawidgets_changed)};
+      #@$self{ keys %arg } = values(%arg);
+        my $arg_holder_ref = { 
+        rec_count_label => \$widgets{ $ido}->{rec_count_label},
+        status_label => \$widgets{ $ido}->{status_label},
+        rec_spinner => \$widgets{ $ido}->{rec_spinner},
+        null_string => \$widgets{ $ido }->{ null_string },
+        on_change => \$events{ $ido }->{on_change},
+        data_lock => \$events{ $ido }->{data_lock},
+        datawidgets_changed => \%ecols,
+
+    } ;
+      for my $name (keys %{$arg_holder_ref}){
+        next unless defined ($arg{$name});
+
+        if (ref $arg_holder_ref->{$name} eq "HASH"){
+           $arg_holder_ref->{$name}->{ $ido } = $arg{$name};
+
+        } 
+        #elsif (ref $arg_holder_ref->{$name} eq "ARRAY") {
+
+        #} 
+        else {
+            ${ $arg_holder_ref->{$name} } =  $arg{$name};
+        
+        }
+    
+    }
+    # bless $self, $class;
 
     #$self->{cols} = [];
     $self->_init;
     $self->SUPER::_init;
+=for comment
     my @dates;
 
-    $self->{subform} = [];
+    # $self->{subform} = [];
 
     #my %formatters_db;
     #my %formatters_f;
     # $self->{dates_formatted} = \(keys %{$self->{date_formatters}});
     foreach my $v ( keys %{ $self->{date_formatters} } ) {
-        $self->{log}->debug( "** " . $v . " **" );
+        $log{ $id }->debug( "** " . $v . " **" );
         push @dates, $v;
     }
     $self->{dates_formatted} = \@dates;
@@ -127,8 +179,9 @@ sub new {
 
     $self->{pos2del}   = [];
     $self->{inserting} = 0;
-    return $self;
 
+=cut
+        return $self;
 }    #new
 
 sub _get_setter { my $self = shift; return %setter; }
@@ -137,45 +190,51 @@ sub _get_getter { my $self = shift; return %getter; }
 sub _init {
 
     my ($self) = @_;
-    $self->{painting} = 1;
+     my $id = id $self;
+    $self->_painting(1);
 
 # get a ref to the Gtk widget used for the record spinner or if the id has been guiven, get the ref via the builder
-    $self->{rec_spinner} = (
-        ref $self->{rec_spinner}
-        ? $self->{rec_spinner}
-        : $self->{builder}->get_object( $self->{rec_spinner} ) );
-    $self->{rec_count_label} = (
-        ref $self->{rec_count_label}
-        ? $self->{rec_count_label}
-        : $self->{builder}->get_object( $self->{rec_count_label} ) );
-    $self->{status_label} = (
-        ref $self->{status_label}
-        ? $self->{status_label}
-        : $self->{builder}->get_object( $self->{status_label} ) );
+    $widgets{ $id }->{rec_spinner} = (
+        ref $widgets{ $id }->{rec_spinner}
+        ? $widgets{ $id }->{rec_spinner}
+        : $self->_builder()->get_object( $widgets{ $id }->{rec_spinner} ) );
+    $widgets{ $id }->{rec_count_label} = (
+        ref $widgets{ $id }->{rec_count_label}
+        ? $widgets{ $id }->{rec_count_label}
+        : $self->_builder()->get_object( $widgets{ $id }->{rec_count_label} ) );
+    $widgets{ $id }->{status_label} = (
+        ref $widgets{ $id }->{status_label}
+        ? $widgets{ $id }->{status_label}
+        : $self->_builder()->get_object( $widgets{ $id }->{status_label} ) );
 
-    $self->{log} = Log::Log4perl->get_logger(__PACKAGE__);
-    $self->{log}->debug(" ** New Form object ** ");
-    $self->{changed} = 0;
+    $log{ $id } = Log::Log4perl->get_logger(__PACKAGE__);
+    $log{ $id }->debug(" ** New Form object ** ");
+    $self->_changed(0);
+=for comment
     if ( !defined $self->{cols} ) {
         my @col = $self->{dman}->get_field_names;
         $self->{cols} = \@col;
-        $self->{log}->debug( "_init cols: " . join( " ", @col ) );
+        $log{ $id }->debug( "_init cols: " . join( " ", @col ) );
     }
-    $self->{hecols} = {};
-    if ( defined $self->{ecols} ) {
-        my %is_ecol;
-        my @fields = keys %{ $self->{ecols} };
-        for (@fields) { $is_ecol{$_}++; }
-        $self->{hecols} = \%is_ecol;
-        $self->{log}->debug( "datawidgets_changed: " . join( " ", @fields ) );
+=cut
+    # $self->{hecols} = {};
+    #  if ( defined $ecols{ $id } ) {
+    if ( $ecols{ $id } ) {
+        # my %is_ecol;
+        my @fields = keys %{ $ecols{ $id } };
+        for (@fields) { 
+            ${ $is_ecols{ $id } }{$_}++; 
+        }
+        #  $self->{hecols} = \%is_ecol;
+        $log{ $id }->debug( "datawidgets_changed: " . join( " ", @fields ) );
 
     }
 
     $self->_bind_on_changed;
     $self->_set_recordspinner;
     croak __PACKAGE__ . ": a data manager is required"
-      unless ( defined $self->{dman} );
-    $self->{dman}->set_row_pos(0);
+      unless ( defined $self->_dman );
+    $self->_dman->set_row_pos(0);
 }
 
 #dman must contains all the rows
@@ -193,8 +252,8 @@ sub _init {
 sub add_combo_values {
 
     my ( $self, $w, $aref, $name ) = @_;
-
-    # $self->{log}->debug( "build_list $name\n");
+ my $id = id $self;
+    # $log{ $id }->debug( "build_list $name\n");
     my $wref       = ref $w;
     my @supp_class = (qw/Wx::ComboBox Wx::ListBox/);
     my %supported  = map { $_ => 1 } @supp_class;
@@ -221,7 +280,7 @@ sub add_combo_values {
             if ( $col_i == 0 ) {
 
 #push @ids, $row->[$i];
-# $self->{log}->debug("add_combo_values val:" . $row->[$col_i] . " index: " . $i_pos);
+# $log{ $id }->debug("add_combo_values val:" . $row->[$col_i] . " index: " . $i_pos);
                 $ids{$i_pos} = $row->[$col_i];
                 $idpos{ $row->[$col_i] } = $i_pos;
 
@@ -236,11 +295,13 @@ sub add_combo_values {
         $w->Append($display);
         $i_pos++;
     }
-    $self->{datawidgetsValue}->{$name} = \%ids;
-    $self->{datawidgetsPos}->{$name}   = \%idpos;
+    # $self->{datawidgetsValue}->{$name} = \%ids;
+    $datawidgets_value{ $id }->{$name} =\%ids;
+    #$self->{datawidgetsPos}->{$name}   = \%idpos;
+    $datawidgets_pos{ $id }->{$name} = \%idpos;
 
-    #$self->{log}->debug("dwV: \n" . Dumper(%ids));
-    #$self->{log}->debug("dvP: \n" . Dumper(%idpos));
+    #$log{ $id }->debug("dwV: \n" . Dumper(%ids));
+    #$log{ $id }->debug("dvP: \n" . Dumper(%idpos));
 
 }
 
@@ -251,6 +312,7 @@ sub add_combo {
 
     #my ($self, $req)=@_;
     my $self = shift;
+    my $id = id $self;
     my %def = ( init => 1 );
     my %h;
     my $req = ( ref $_[0] eq "HASH" ) ? $_[0] : ( %h = ( %def, @_ ) ) && \%h;
@@ -275,29 +337,29 @@ sub add_combo {
     if ( $$req{builder} && ( ref $self eq "" ) ) {    #static init
 
         $self = {};
-        $self->{builder} = $$req{builder};
+        $self->_builder($$req{builder});
 
-        $self->{log} = Log::Log4perl->get_logger(__PACKAGE__);
+        $log{ $id } = Log::Log4perl->get_logger(__PACKAGE__);
 
-        my $w = $self->{builder}->get_object( $combo->{id} );
+        my $w = $self->_builder()->get_object( $combo->{id} );
         if ($w) {
 
             #my $name = $w->get_name;
             my $name = ref $w;
 
             #$name =~s/::/_/g;
-            $self->{log}
+            $log{ $id }
               ->debug( "name: " . $name . "  widget->GetName: " . $w->GetName );
-            $self->{datawidgets}->{ $combo->{id} }     = $w;
-            $self->{datawidgetsName}->{ $combo->{id} } = $name;
+            $self->_datawidgets($combo->{id}, $w);
+            $self->_datawidgetsName( $combo->{id}, $name);
         } else {
-            $self->{log}->debug( "cols: " . join( " ", @cols ) );
+            $log{ $id }->debug( "cols: " . join( " ", @cols ) );
             confess "no widget found for combo " . $combo->{id};
         }
     }
 
-    $self->{log}->debug( "cols: " . join( " ", @cols ) );
-    my $w = $self->{datawidgets}->{ $combo->{id} };
+    $log{ $id }->debug( "cols: " . join( " ", @cols ) );
+    my $w = $self->_datawidgets( $combo->{id} );
     confess( 'no widget found for combo ' . $combo->{id} ) unless ($w);
 
     $w->Clear;
@@ -317,7 +379,7 @@ sub add_combo {
     my $last;
 
     #Name is the Wx::XXXX kind of object
-    my $name = $self->{datawidgetsName}->{ $combo->{id} };
+    my $name = $self->_datawidgetsName($combo->{id});
 
     #my %idpos;
     my $itemCol;
@@ -369,7 +431,7 @@ sub add_combo {
                     confess("add_combo: $name is not supported");
                 }
 
-      #else {$self->{log}->error("add_combo: $name is not supported" ); return;}
+      #else {$log{ $id }->error("add_combo: $name is not supported" ); return;}
 
             } else {
 
@@ -385,12 +447,12 @@ sub add_combo {
                     #$w->SetString($i, $value);
                     $w->Append($value);
 
-   #$self->{log}->debug("add_combo col $column index:  $i value: $value");
-   #$self->{log}->debug("add_combo: ". $combo->{id} . " row: $i value: $value");
+   #$log{ $id }->debug("add_combo col $column index:  $i value: $value");
+   #$log{ $id }->debug("add_combo: ". $combo->{id} . " row: $i value: $value");
                 }
             }
 
-#$self->{log}->debug("add combo: column $column index $i value $value cd: ". ($w->GetClientData($i) ? $w->GetClientData($i) : "undef"));
+#$log{ $id }->debug("add combo: column $column index $i value $value cd: ". ($w->GetClientData($i) ? $w->GetClientData($i) : "undef"));
         }    #for rows
 
         #$column = ($column == 0 ? 1 : 0);
@@ -401,26 +463,28 @@ sub add_combo {
 
         #$w->SetClientData(@idval);
         #$self->{datawidgetsValue}->{ $combo->{id} } = \%idpos;
-        $self->{datawidgetsValue}->{ $combo->{id} } = \%ids;
-        $self->{datawidgetsPos}->{ $combo->{id} }   = \%idpos;
+        #$self->{datawidgetsValue}->{ $combo->{id} } = \%ids;
+        $datawidgets_value{$id }->{ $combo->{id} } = \%ids;
+        #$self->{datawidgetsPos}->{ $combo->{id} }   = \%idpos;
+        $datawidgets_pos{ $id }->{ $combo->{id} }   = \%idpos;
         my $href = { fields => $combo->{fields}, dman => $combo->{dman}, };
-        $self->{combo}->{ $combo->{id} } =
-          { dman => $combo->{dman}, fields => \@cols };
+        # $self->{combo}->{ $combo->{id} } ={ dman => $combo->{dman}, fields => \@cols };
+        $combos{ $id } ={ dman => $combo->{dman}, fields => \@cols };
 
     }
 
-    $self->{log}->debug( "add_combo: " . $last . " rows added" );
+    $log{ $id }->debug( "add_combo: " . $last . " rows added" );
 
 #crase le listener mis dans bind_changed
-#EVT_LIST_ITEM_SELECTED($self->{builder}->get_object('mainwindow'), $w, sub {$self->_on_selected($combo->{id}, @_)});
+#EVT_LIST_ITEM_SELECTED($self->_builder()->get_object('mainwindow'), $w, sub {$self->_on_selected($combo->{id}, @_)});
     if ( $combo->{init} && $w->GetWindowStyleFlag & wxTE_PROCESS_ENTER ) {
-        $self->{log}->debug( "binding text enter for combo " . $combo->{id} );
-        EVT_TEXT_ENTER( $self->{builder}->frame,
+        $log{ $id }->debug( "binding text enter for combo " . $combo->{id} );
+        EVT_TEXT_ENTER( $self->_builder()->frame,
             $w, sub { $self->_on_combo_newval( $combo->{id}, @_ ) } );
     }
 
     #	if ($self->{datawidgetsName}->{$combo->{id}} eq "GtkComboBoxEntry" ){
-    if ( $self->{datawidgetsName}->{ $combo->{id} } eq "Wx::ListCtrl" ) {
+    if ( $self->_datawidgetsName($combo->{id}) eq "Wx::ListCtrl" ) {
 
         #if ( ! $self->{combos_set}->{$combo->{id}} ) {
         #$w->set_text_column( 1 );
@@ -449,27 +513,27 @@ sub add_radio_button {
 #bind an onchanged sub with each modification of the datafields
 sub _bind_on_changed {
     my $self = shift;
-
+ my $ido = id $self;
     # my @cols = $self->{dman}->get_field_names;
-    foreach my $id ( @{ $self->{cols} } ) {
-        my $w = $self->{builder}->get_object($id);
-        $self->{log}->debug( "bind_on_changed looking for widget " . $id );
+    foreach my $id ( @{ $self->_cols } ) {
+        my $w = $self->_builder()->get_object($id);
+        $log{ $ido }->debug( "bind_on_changed looking for widget " . $id );
         if ($w) {
 
             # my $name = $w->GetName;
             my $name = ref $w;
 
             #$name =~s/::/_/g;
-            $self->{datawidgets}->{$id}     = $w;
-            $self->{datawidgetsName}->{$id} = $name;
+            $self->_datawidgets( $id, $w);
+            $self->_datawidgetsName($id, $name);
 
             if ( ref( $signals{$name} ) eq "CODE" ) {
 
                 my $coderef = $signals{$name};
 
-#&$coderef($self->{builder}->get_object('mainwindow'), $w, sub{$self->_changed($id, $w, @_)});
-                &$coderef( $self->{builder}->frame,
-                    $w, sub { $self->_changed( $id, $w, @_ ) } );
+#&$coderef($self->_builder()->get_object('mainwindow'), $w, sub{$self->_changed($id, $w, @_)});
+                &$coderef( $self->_builder()->frame,
+                    $w, sub { $self->_change_values( $id, $w, @_ ) } );
 
 #mettre le widget lui meme comme premier arg entraine que la fonction est toujours executee
 #meme si une autre fonction est attachee apres. Si le premier arg est toujours le meme, c'est la derniere
@@ -477,13 +541,13 @@ sub _bind_on_changed {
 #&$coderef($w, $w, sub{$self->_changed($id, $w, @_)});
 
             }
-            $self->{log}->debug("bind  $name $id with self->changed");
+            $log{ $ido }->debug("bind  $name $id with self->changed");
 
     #$w->signal_connect_after( $signals{$name} => sub{ $self->_changed( $id )});
             die("signal undef for $name") unless ( $signals{$name} );
 
         } else {
-            $self->{log}->debug(" ... not found ");
+            $log{ $ido }->debug(" ... not found ");
         }
     }
 
@@ -493,96 +557,103 @@ sub _bind_on_changed {
 # Place
 sub _set_recordspinner {
     my $self = shift;
-    $self->{log}->debug("set_recordspinner");
+     my $id = id $self;
+    $log{ $id }->debug("set_recordspinner");
 
-    # die unless($self->{rec_spinner});
+    # die unless($widgets{ $id }->{rec_spinner});
     my $coderef;
-    if ( $self->{rec_spinner} ) {
+    if ( $widgets{ $id }->{rec_spinner} ) {
         $coderef = sub {
-            my $pos = $self->{rec_spinner}->GetValue - 1;
-            $self->{log}->debug( "rs_value changed will move to " . $pos );
+            
+            my $pos = $widgets{ $id }->{rec_spinner}->GetValue - 1;
+            $log{ $id }->debug( "rs_value changed will move to " . $pos );
 
-            #self->{rec_spinner}->signal_handler_block( $coderef );
+            #widgets{ $id }->{rec_spinner}->signal_handler_block( $coderef );
             #$self->move( undef, $pos);
 
             #confess ( $pos, " already seen") if ($pos_seen{$pos});
             # $pos_seen{$pos}++;
 
-            if ( $self->{auto_apply} && $self->has_changed ) { $self->apply; }
+            if ( $self->_auto_apply && $self->has_changed ) { $self->apply; }
 
             # done in display data
             #$self->{dman}->set_row_pos($pos);
             $self->_display_data($pos);
 
-            #$self->{rec_spinner}->signal_handler_unblock( $coderef );
+            #$widgets{ $id }->{rec_spinner}->signal_handler_unblock( $coderef );
             # return 1;
         };
 
-        EVT_SPINCTRL( $self->{rec_spinner}, $self->{rec_spinner}, $coderef );
+        EVT_SPINCTRL( $widgets{ $id }->{rec_spinner}, $widgets{ $id }->{rec_spinner}, $coderef );
 
 # With this EVT_TEXT marcro, a change done with ->insert calls the code above
-# EVT_TEXT($self->{rec_spinner}, $self->{rec_spinner}, $coderef);
+# EVT_TEXT($widgets{ $id }->{rec_spinner}, $widgets{ $id }->{rec_spinner}, $coderef);
 # from http://docs.wxwidgets.org/trunk/classwx_spin_ctrl.html
 # if the user modifies the text in the edit part of the spin control directly, the EVT_TEXT is generated,
 # like for the wxTextCtrl. When the use enters text into the text area, the text is not validated until the control loses
 # focus (e.g. by using the TAB key). The value is then adjusted to the range and a wxSpinEvent sent
 # then if the value is different from the last value sent.
-        $self->{rs_value_changed_signal} = $coderef;
-        $self->{log}->debug("recordspinner set");
+        #$self->{rs_value_changed_signal} = $coderef;
+        $events{ $id }->{rs_value_changed_signal} = $coderef;
+        weaken  $events{ $id }->{rs_value_changed_signal};
+        $log{ $id }->debug("recordspinner set");
     }
 
 }
 
 sub _set_rs_range {
     my ( $self, $first, $last ) = @_;
-
-    $self->{log}->debug( "set_rs_range  first : " . $first );
-    my $rowcount = $self->{dman}->row_count;
-    if ( $self->{rec_spinner} ) {
-        $self->{rec_spinner}->SetRange( $first, $last );
+ my $id = id $self;
+    $log{ $id }->debug( "set_rs_range  first : " . $first );
+    my $rowcount = $self->_dman->row_count;
+    if ( $widgets{ $id }->{rec_spinner} ) {
+        $widgets{ $id }->{rec_spinner}->SetRange( $first, $last );
     }
 
 #Note that this function will not generate the wxEVT_COMMAND_TEXT_UPDATED event.
-#$self->{rec_count_label}->ChangeValue(" / " . $self->{dman}->row_count);
-    $self->{rec_count_label}->SetLabel( " / " . $self->{dman}->row_count );
+#$widgets{ $id }->{rec_count_label}->ChangeValue(" / " . $self->{dman}->row_count);
+    $widgets{ $id }->{rec_count_label}->SetLabel( " / " . $rowcount );
     return 1;
 
 }
 
 sub _set_entry {
     my ( $self, $w, $x, $id ) = @_;
+     my $ido = id $self;
     if ( defined $x ) {
-        $self->{log}->debug( "set_entry: " . $x );
+        $log{ $ido }->debug( "set_entry: " . $x );
 
 #$w->set_text( $x ) ;
 # SetValue generates a wxEVT_TEXT event. To avoid this you can use ChangeValue() instead.
-#$self->{hecols}->{$id} ? $w->SetValue($x) && $self->{log}->debug("SetValue called") : $w->ChangeValue($x);
+#$self->{hecols}->{$id} ? $w->SetValue($x) && $log{ $id }->debug("SetValue called") : $w->ChangeValue($x);
         confess("id undef") unless ( defined $id );
-        if ( $self->{hecols}->{$id} ) {
+        #if ( $self->{hecols}->{$id} ) {
+        if ( $is_ecols{ $ido }->{$id} ) {
             $w->SetValue($x);
 
-            #$self->{log}->debug("SetValue called");
+            #$log{ $id }->debug("SetValue called");
         } else {
             $w->ChangeValue($x);
 
-            #$self->{log}->debug("ChangeValue called");
+            #$log{ $id }->debug("ChangeValue called");
         }
     } else {
-        $self->{log}->debug( "set_entry: text entry undef " . $w->GetName );
+        $log{ $ido }->debug( "set_entry: text entry undef " . $w->GetName );
 
         #$w->set_text("");
-        $self->{hecols}->{$id} ? $w->Clear : $w->ChangeValue("");
+        #$self->{hecols}->{$id} ? $w->Clear : $w->ChangeValue("");
+        $is_ecols{ $ido }->{$id} ? $w->Clear : $w->ChangeValue("");
 
         #$w->ChangeValue("");
     }
 
-#$self->{log}->debug("hecols : ". ($self->{hecols}->{$id} ? $self->{hecols}->{$id} : "undef") . " for " . $id );
+#$log{ $id }->debug("hecols : ". ($self->{hecols}->{$id} ? $self->{hecols}->{$id} : "undef") . " for " . $id );
 
 }
 
 sub _set_textentry {
     my ( $self, $w, $x, $id ) = @_;
-    $self->{log}->debug("set_textentry text entry undef") if ( !defined $x );
+    $log{ id $self }->debug("set_textentry text entry undef") if ( !defined $x );
     $w->WriteText( $x || "" );
 
 }
@@ -592,10 +663,10 @@ sub _set_textentry {
 # $id corresponding index
 sub _set_combo {
     my ( $self, $w, $x, $id ) = @_;
-
+ my $ido = id $self;
 #return unless (defined $x); the combo must be unselect from the preceding value
-    my $name = $self->{datawidgetsName}->{$id};
-    $self->{log}->debug( "set_combo value "
+    my $name = $self->_datawidgetsName($id);
+    $log{ $ido }->debug( "set_combo value "
           . ( defined $x ? $x : " undef" )
           . " widget: "
           . $name . " id: "
@@ -609,11 +680,13 @@ sub _set_combo {
         $w->SetItemState( $item, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED );
     } elsif ( $name eq "Wx::ComboBox" || $name eq "Wx::ListBox" ) {
         confess( "undef datawidgetsPos  for ", $id )
-          unless ( $self->{datawidgetsPos}->{$id} );
-        my %idpos = %{ $self->{datawidgetsPos}->{$id} };
+        unless ( $datawidgets_pos{ $ido }->{$id} );
+        #unless ( $self->{datawidgetsPos}->{$id} );
+        #my %idpos = %{ $self->{datawidgetsPos}->{$id} };
+        my %idpos = %{ $datawidgets_pos{ $ido }->{$id} };
 
         my $i = $idpos{$x} if ( defined $x );
-        $self->{log}
+        $log{ $ido }
           ->debug( "_set_combo pos is " . ( defined $i ? $i : " undef" ) );
 
         # $w->SetSelection(wxNOT_FOUND);
@@ -623,7 +696,7 @@ sub _set_combo {
             $w->SetSelection(wxNOT_FOUND);
         }
 
-        # $self->{log}->debug($w->GetStringSelection);
+        # $log{ $id }->debug($w->GetStringSelection);
 
     }
 }
@@ -638,11 +711,12 @@ sub _set_check {
 # $field[0] is the field name of the values returned from the combo
 sub _on_combo_newval {
     my ( $self, $comboid, $frame, $event ) = @_;
+     my $id = id $self;
     my $value = $event->GetString;
-    $self->{log}->debug( "combo_newval: " . $value );
+    $log{ $id }->debug( "combo_newval: " . $value );
     $value = ( $value eq "" ? undef : $value );
     return unless ( defined $value );
-    my $href   = $self->{combo}->{$comboid};
+    my $href   = $combos{ $id }->{$comboid};
     my $dman   = $href->{dman};
     my @fields = @{ $href->{fields} };
     $dman->new_row;
@@ -660,66 +734,71 @@ sub _on_combo_newval {
 
 sub _get_combobox_selectedvalue {
     my ( $self, $id ) = @_;
+     my $ido = id $self;
     confess("id undef") unless ( defined $id );
-    my $w    = $self->{datawidgets}->{$id};
-    my $name = $self->{datawidgetsName}->{$id};
+   
+    my $w    = $self->_datawidgets( $id );
+    my $name = $self->_datawidgetsName( $id );
     my $x;
 
     if ( $name eq "Wx::ListCtrl" ) {
         $x =
-          (   $self->{datawidgetsValue}->{$id}
-            ? $self->{datawidgetsValue}->{$id}
+          (   $datawidgets_value{ $ido }->{$id}
+            ? $datawidgets_value{ $ido }->{$id}
             : undef );
     } elsif ( $name eq "Wx::ComboBox" || $name eq "Wx::ListBox" ) {
 
         my $pos = $w->GetSelection();
 
         #die ((($pos == wxNOT_FOUND) ? "not": "") . " found");
-        #$self->{log}->debug ("id : ", $id, " pos ", $pos);
-        #$self->{log}->debug(  $self->{datawidgetsValue}->{$id}->{$pos});
+        #$log{ $id }->debug ("id : ", $id, " pos ", $pos);
+        #$log{ $id }->debug(  $self->{datawidgetsValue}->{$id}->{$pos});
 
         $x =
           ( $pos == wxNOT_FOUND )
           ? undef
-          : $self->{datawidgetsValue}->{$id}->{$pos};
+          : $datawidgets_value{ $ido }->{$id}->{$pos};
 
     }
-    $self->{log}->debug( "_get_combobox_selectedvalue: found "
+    $log{ $ido }->debug( "_get_combobox_selectedvalue: found "
           . ( defined $x ? $x : " undef" ) );
     return $x;
 }
 
 sub _set_spinbutton {
     my ( $self, $w, $x ) = @_;
-    if ( $self->getID($w) eq $self->getID( $self->{rec_spinner} ) ) {
-        $self->{log}->debug("Found record_spinner... leaving");
+     my $id = id $self;
+=for comment
+    if ( $self->getID($w) eq $self->getID( $widgets{ $id }->{rec_spinner} ) ) {
+        $log{ $id }->debug("Found record_spinner... leaving");
         return;
     }
+=cut
     $w->SetValue( $x || 0 );
 
 }
 
-sub _changed {
+sub _change_values {
     my ( $self, $fieldname, $w, $frame, $event ) = @_;
-
+ my $id = id $self;
     #my $name = ref $w;
-    my $name = $self->{datawidgetsName}->{$fieldname};
-    $self->{log}->debug("self->changed for $fieldname ($name)");
+    my $name = $self->_datawidgetsName($fieldname);
+    $log{ $id }->debug("self->changed for $fieldname ($name)");
+    die ("$fieldname without type") unless defined $name;
     if ( $name eq "Wx::ListCtrl" ) {
         print "text: ", $event->GetText, "Index: ", $event->GetIndex, "Data: ",
           $event->GetData, "\n";
-        $self->{datawidgetsValue}->{$fieldname} = $event->GetText;
+        $datawidgets_value{ $id }->{$fieldname} = $event->GetText;
 
     }
-    if ( $self->{hecols}->{$fieldname} ) {
-        my $coderef = $self->{ecols}->{$fieldname};
+    if ( $is_ecols{ $id }->{$fieldname} ) {
+        my $coderef = $ecols{ $id }->{$fieldname};
         &$coderef( $w, $event );
-
     }
-    if ( !$self->{painting} ) {
-        $self->{changed} = 1;
-        if ( $self->{on_changed} ) {
-            $self->{on_changed}();
+    if ( !$self->_painting ) {
+        $self->_changed(1);
+        if ( $events{$id}->{on_change} ) {
+            $events{$id}->{on_change}();
         }
         $self->_set_record_status_label;
     }
@@ -729,15 +808,15 @@ sub _changed {
 
 sub _on_selected {
     my ( $self, $id, $list, $event ) = @_;
-
+ my $ido = id $self;
 #print Dumper $self;
 #print "_on_selected: name: ", ref $list, "\n";
 #print "text: " , $event->GetText, "Index: ", $event->GetIndex, "Data: ", $event->GetData, "\n";
-    if ( $self->{datawidgetsName}->{$id} eq "Wx::ListCtrl" ) {
-        $self->{datawidgetsValue}->{$id} = $event->GetText;
+    if ( $self->_datawidgetsName( $id  ) eq "Wx::ListCtrl" ) {
+        $datawidgets_value{ $ido }->{$id} = $event->GetText;
     } else {
         croak(  "_on_selected not implemented for $id ("
-              . $self->{datawidgetsName}->{$id}
+              . $self->_datawidgetsName($id)
               . ")" );
     }
 
@@ -746,18 +825,18 @@ sub _on_selected {
 sub _set_record_status_label {
 
     my $self = shift;
+ my $id = id $self;
+ # $log{ $id }->debug("set_record_satus_label changed is " . $self->_changed);
 
- # $self->{log}->debug("set_record_satus_label changed is " . $self->{changed});
+    if ( $widgets{ $id }->{status_label} ) {
+        if ( $events{ $id }->{data_lock} ) {
+            $widgets{ $id }->{status_label}->SetLabel("Locked");
+        } elsif ( $self->_changed ) {
 
-    if ( $self->{status_label} ) {
-        if ( $self->{data_lock} ) {
-            $self->{status_label}->SetLabel("Locked");
-        } elsif ( $self->{changed} ) {
-
-            $self->{status_label}->SetLabel("Changed");
+            $widgets{ $id }->{status_label}->SetLabel("Changed");
 
         } else {
-            $self->{status_label}->SetLabel("Synchronized");
+            $widgets{ $id }->{status_label}->SetLabel("Synchronized");
         }
     }
 }
@@ -838,7 +917,7 @@ C<time_zone> and C<locale> are needed by Date::Time::Strptime.
 
 
 
-To display new rows on a bound subform, connect the on_changed event to the field of the primary key in the main form.
+To display new rows on a bound subform, connect the on_change event to the field of the primary key in the main form.
 In this sub, call a sub to synchonize the form:
 
 In the main form:
@@ -966,7 +1045,7 @@ C<$builder> is an object that must have a C<get_object($object_name)> which is a
    sub get_object {
 	my ($self, $id) = @_;
 	my $w = Wx::Window::FindWindowByName($id, $self->{frame});
-	$self->{log}->debug("get_object: $id not found") unless (defined $w);
+	$log{ $id }->debug("get_object: $id not found") unless (defined $w);
 	return $w;
     }
 
